#####################################################################
#                                                                   #
# ALL RIGHTS RESERVED, COPYRIGHT (C) SHAVLIK TECHNOLOGIES, 2017     #
#                                                                   #
# DISCLAIMERS / LIMITATIONS                                         #
# By utilizing this product, you acknowledge that Shavlik           #
# Technologies (Shavlik) has provided this item on a "as is, where  #
# is"  basis., without warranty of any kind, express or implied,    #
# including implied warranties of non-infringement, merchantability #
# and fitness for a particular purpose.  Shavlik does not warrant   #
# that the product will meet your requirements, operate in          #
# combination with other software, or be uninterrupted or error     #
# free.  In no event shall Shavlik be liable for any lost revenue,  #
# profit, or data, or for direct, special, indirect, consequential, #
# incidental, or punitive damages however caused and regardless of  #
# the theory of liability, arising out of the use of or inability   #
# to use the product even if Shavlik has been advised of the        #
# possibility of such damages.                                      #
#####################################################################

<#
.SYNOPSIS
	Imports the certificate located in the Certifcate folder.

.DESCRIPTION
	Imports the certificate located in the Certifcate folder.
	
	The certificate file name has to be in the following format and the Description cannot comtain _:
	
	Description_CertificateStore.cer
	
	Example:
		DigiCert Assured ID Root CA_Root.cer

.PARAMETER $computerName
	The machine name or IP Address to fix

.PARAMETER $machineListFilePath
	The fully qualified Machine name list file path. Example, "c:\users\user name\desktop\MachineList.txt"
	The file path has to be quoted.
	The Machine List file has to contain one machine name or IP Address per line. See example MachineList.txt file.
#>

param
(
	[string]$computerName = $env:COMPUTERNAME,
	[string]$machineListFilePath = ""
)

cls
$ScriptVersion = "1.0.0.3"
$computerName = $computerName.Trim()
$machineListFilePath = $machineListFilePath.Trim()
$currentFolder = split-path $SCRIPT:MyInvocation.MyCommand.Path -parent
$scriptFileNameWithoutExt = [System.IO.Path]::GetFileNameWithoutExtension($MyInvocation.MyCommand.Name)

$certFileExt = ".cer"
$certFolder = [System.IO.Path]::Combine($currentFolder, "Certificates")
[System.Security.Cryptography.X509Certificates.X509FindType]$findType = [System.Security.Cryptography.X509Certificates.X509FindType]::FindByThumbprint
$validCertStoresWeCareAbout = 
	"TrustedPublisher", 
	"ClientAuthIssuer",
	"Trust",
	"Root",
	"TrustedDevices",
	"CA",
	"Request",
	"AuthRoot"

#####################################
#			Tracing
#####################################
$Script:traceFile = "$currentFolder\${scriptFileNameWithoutExt}_Trace.log"
function ST-Trace
{
	param(	[Parameter(Position=0,Mandatory=$true)] [object]$private:message,
			[Parameter(Position=1,Mandatory=$false)] [object]$private:value,
			[Parameter(Position=2,Mandatory=$false)] [bool]$private:showMsg = $true)
		
	$private:currentDateTime = [DateTime]::UtcNow.ToString("yyyy-MM-ddTHH:mm:ss.fffffff") + "z"
	
	$private:finalMessage = "$private:currentDateTime " + $private:message + $private:value
	Add-Content $Script:traceFile $private:finalMessage
	
	if ($private:showMsg)
	{
		Write-Host "$private:message$private:value"
	}
}

function Validate-CertFile([array] $temp)
{
	$validErrors = @()
	if ($validCertStoresWeCareAbout -inotcontains $temp[1])	{ $validErrors += ($temp[0] + " contains an invalid certificate store: " + $temp[1] + "`r`n`r`n") }
	if ($temp[2] -eq $null) { $validErrors += ("Unable to load " + $temp[0] + "'s certificate ") }
	else { if ($temp[2].Thumbprint.Length -ne 40)	{ $validErrors += ($temp[0] + " thumbprint length does not equal 40: " + $temp[2].Thumbprint + "(Length: " + $temp[2].Thumbprint.Length + ")" + "`r`n`r`n") } }
	
	if ($validErrors.Count -ne 0)
	{
		ST-Trace $validErrors
		return $false
	}
	return $true
}

function Get-Cert([string] $storeNamePath, [string] $findValue)
{
	try
	{
	    $store = new-object System.Security.Cryptography.X509Certificates.X509Store($storeNamePath, [System.Security.Cryptography.X509Certificates.StoreLocation]::LocalMachine)
	    $store.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadOnly)
		$certs = $store.Certificates.Find($findType, $findValue, $true)
		if ($certs.Count -gt 0)
		{
			return $true;
		}
	}
	catch [Exception] 
	{
		ST-Trace "Get-Cert `$findType: $findType, `$findValue: $findValue" $_.Exception
	}
	finally
	{
		if ($store -ne $null)
		{
			$store.Close()
		}
	}
	return $false
}

function Import-Cert([string] $storeNamePath, [string] $description, [System.Security.Cryptography.X509Certificates.X509Certificate2] $cert)
{
	try
	{		
	    $store = new-object System.Security.Cryptography.X509Certificates.X509Store($storeNamePath, [System.Security.Cryptography.X509Certificates.StoreLocation]::LocalMachine)
	    $store.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadWrite)
		$store.Add($cert)

		if (Get-Cert $storeNamePath $cert.Thumbprint)
		{
			ST-Trace "`tSuccessfully added $description"
		}
		else
		{
			ST-Trace "`tFailed to add $description"
		}
	}
	finally
	{
		if ($store -ne $null)
		{
			$store.Close()
		}
	}
}

function ProcessMachine([string] $computerName, [array] $certificates)
{
	try
	{
		$Error.Clear()
		ST-Trace "******************************"
		ST-Trace "   Processing $computerName - Started"
		ST-Trace "******************************"
		ST-Trace ""
		
		foreach ($value in $certificates)
		{
			$description = $value[0]
			$storeNamePath = "\\$computerName\" + $value[1]
			ST-Trace "`tAttempting to import: $description"
			ST-Trace "`t`$storeNamePath: $storeNamePath" "" $false
			if (Get-Cert $storeNamePath $value[2].Thumbprint)
			{
				ST-Trace "`t$description was found. Not importing certificate"
			}
			else
			{
				ST-Trace "`t$description was NOT found. Importing certificate"
				Import-Cert $storeNamePath $description $value[2]
			}
			ST-Trace ""
		}
	}
	catch [Exception]
	{
		$private:e = $_.Exception
		do
		{
			ST-Trace "Error: " $private:e
			$private:e = $private:e.InnerException
		}
		while ($private:e -ne $null)
	}
	finally
	{
		ST-Trace "******************************"
		ST-Trace "   Processing $computerName - End"
		ST-Trace "******************************"
		ST-Trace ""
	}
}

#####################################
#			Start Script
#####################################
try
{
	ST-Trace "#######################################" "" $false
	ST-Trace "Version: $ScriptVersion" "" $false
	ST-Trace "" "" $false
	ST-Trace "`$computerName: $computerName" "" $false
	ST-Trace "`$machineListFilePath: $machineListFilePath" "" $false
	ST-Trace "`$certFolder: $certFolder" "" $false
	ST-Trace "#######################################" "" $false
	ST-Trace "" "" $false
	
	$private:certificates = @()
	$certFiles = @(Get-ChildItem $certFolder | where {$_.Extension -match $certFileExt})
	foreach ($certfile in $certFiles)
	{
		$temp = @()
		$temp += $certfile.BaseName.Split("_") | foreach {$_.Trim()}
		$temp += New-Object System.Security.Cryptography.X509Certificates.X509Certificate2 ($certfile.FullName.Trim())
		
		if (Validate-CertFile $temp)
		{
			$private:certificates += , $temp
			ST-Trace "Going to import the following certificate:" "" $false
			ST-Trace ("`tDescription: " + $temp[0]) "" $false
			ST-Trace ("`tCertificate Store: " + $temp[1]) "" $false
			ST-Trace ("`tThumbprint: " + $temp[2].Thumbprint) "" $false
			ST-Trace "" "" $false
		}
	}
	
	if ([string]::IsNullOrEmpty($machineListFilePath))
	{
		ST-Trace "Processing individual Computer" "" $false
		ProcessMachine $computerName $private:certificates
	}
	else
	{
		ST-Trace "Processing a list of machines" "" $false
		if (Test-Path $machineListFilePath)
		{
			foreach ($computerName in Get-Content($machineListFilePath))
			{
				if (![string]::IsNullOrEmpty($computerName.Trim()))
				{
					ProcessMachine $computerName $private:certificates
				}
				else
				{
					ST-Trace "`"$computerName`" is an empty string."
				}
			}
		}
		else
		{
			ST-Trace "$machineListFilePath does not exist."
		}
	}
}
catch [Exception]
{
	$private:e = $_.Exception
	do
	{
		ST-Trace "Error: " $private:e
		$private:e = $private:e.InnerException
	}
	while ($private:e -ne $null)
}
finally
{
	ST-Trace "#######################################" "" $false
	ST-Trace "             End              " "" $false
	ST-Trace "#######################################" "" $false
	ST-Trace "" "" $false
}
# SIG # Begin signature block
# MIINTQYJKoZIhvcNAQcCoIINPjCCDToCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBHJJj13x8bgj3Z
# 6O2JX3eyf/b03zQF7l499+gyal9YA6CCCmowggUwMIIEGKADAgECAhAECRgbX9W7
# ZnVTQ7VvlVAIMA0GCSqGSIb3DQEBCwUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0xMzEwMjIxMjAwMDBa
# Fw0yODEwMjIxMjAwMDBaMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lD
# ZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3
# DQEBAQUAA4IBDwAwggEKAoIBAQD407Mcfw4Rr2d3B9MLMUkZz9D7RZmxOttE9X/l
# qJ3bMtdx6nadBS63j/qSQ8Cl+YnUNxnXtqrwnIal2CWsDnkoOn7p0WfTxvspJ8fT
# eyOU5JEjlpB3gvmhhCNmElQzUHSxKCa7JGnCwlLyFGeKiUXULaGj6YgsIJWuHEqH
# CN8M9eJNYBi+qsSyrnAxZjNxPqxwoqvOf+l8y5Kh5TsxHM/q8grkV7tKtel05iv+
# bMt+dDk2DZDv5LVOpKnqagqrhPOsZ061xPeM0SAlI+sIZD5SlsHyDxL0xY4PwaLo
# LFH3c7y9hbFig3NBggfkOItqcyDQD2RzPJ6fpjOp/RnfJZPRAgMBAAGjggHNMIIB
# yTASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAK
# BggrBgEFBQcDAzB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6Ly9v
# Y3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYDVR0fBHow
# eDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0Rp
# Z2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDBPBgNVHSAESDBGMDgGCmCGSAGG/WwA
# AgQwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAK
# BghghkgBhv1sAzAdBgNVHQ4EFgQUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHwYDVR0j
# BBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQELBQADggEBAD7s
# DVoks/Mi0RXILHwlKXaoHV0cLToaxO8wYdd+C2D9wz0PxK+L/e8q3yBVN7Dh9tGS
# dQ9RtG6ljlriXiSBThCk7j9xjmMOE0ut119EefM2FAaK95xGTlz/kLEbBw6RFfu6
# r7VRwo0kriTGxycqoSkoGjpxKAI8LpGjwCUR4pwUR6F6aGivm6dcIFzZcbEMj7uo
# +MUSaJ/PQMtARKUT8OZkDCUIQjKyNookAv4vcn4c10lFluhZHen6dGRrsutmQ9qz
# sIzV6Q3d9gEgzpkxYz0IGhizgZtPxpMQBvwHgfqL2vmCSfdibqFT+hKUGIUukpHq
# aGxEMrJmoecYpJpkUe8wggUyMIIEGqADAgECAhAEUbq7RDsPjBNjVEBes9UzMA0G
# CSqGSIb3DQEBCwUAMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
# bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0
# IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EwHhcNMTYwNzA4MDAwMDAw
# WhcNMTkwNzE3MTIwMDAwWjBvMQswCQYDVQQGEwJVUzELMAkGA1UECBMCVVQxFTAT
# BgNVBAcTDFNvdXRoIEpvcmRhbjEdMBsGA1UEChMUU2hhdmxpayBUZWNobm9sb2dp
# ZXMxHTAbBgNVBAMTFFNoYXZsaWsgVGVjaG5vbG9naWVzMIIBIjANBgkqhkiG9w0B
# AQEFAAOCAQ8AMIIBCgKCAQEArmM9Io5oJKkmN4/ijiQZvC/uTy8rGthG/om9HCsK
# LpJw+NcHqvrHAvUrpJyOeJ3yJINmqBcS+auJoWkbPeYoeMfpbj1zuMAgJeJCzCaq
# E2rcFRKkeXIoUUmBhhdQ1jvo1K2f/qI4ESbUl56YnIJLF6KHGZwvcCCnaQ/sTUAk
# qUEbJJtRoLEe/Q9fGk9M3ctS4jgomJGI+cfgMxexB9gTZ0DXjuVhJhKGcT9xEe0C
# /dlXHM1iGgoqSVUpXyvUyw+q/wv9nni12Mzwady8mOcg2GSu95ZA48G7wr56VfzL
# +JBrcyD6JyTR4biK1zJ7VKQdt+wLtn357SE4Nn6EnoCBhQIDAQABo4IBxTCCAcEw
# HwYDVR0jBBgwFoAUWsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFB3NCCf8
# HyQ1Ld/S5D3ge5W5umUMMA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEF
# BQcDAzB3BgNVHR8EcDBuMDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20v
# c2hhMi1hc3N1cmVkLWNzLWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNl
# cnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgB
# hv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQ
# UzAIBgZngQwBBAEwgYQGCCsGAQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDov
# L29jc3AuZGlnaWNlcnQuY29tME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5j
# cnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAL8Whe/bvogdD0XmS
# c0dDBKI6PVDpYWA/TwCLyOsZXMrw23Gf5ueXY/OPWiGOr7TmHIHmeaoJX13/kP/b
# 6+n+gFw1tM33c5EhqWITB3KcyC96+H7xb4dFLzVT8gStc/jb9iGCMiv6F0buOIb3
# 52/p/XPF+PWYkCUr1ICsJwhxnv7me1QB4Vi1kKPCUUZQwW1kHvuzHH1p4PbwA2Ol
# 7dOp9aZytj55zmNImI6RPSFjA/bJvG9QcHex7NMPMOGlIljsmXwoikhe4oKRo8sC
# AP/Oss5kcEc2AYdYm9x1cDIxBhvvtZP5r+biOUm0+/Uvp8ERcasvOwY4dCtdD5wO
# su6KKDGCAjkwggI1AgEBMIGGMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMTKERp
# Z2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0ECEARRurtEOw+M
# E2NUQF6z1TMwDQYJYIZIAWUDBAIBBQCggYQwGAYKKwYBBAGCNwIBDDEKMAigAoAA
# oQKAADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4w
# DAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQgKkQk0cKWm61QjLlZa304ElhX
# /YWpuLRtpyTiyktOlgAwDQYJKoZIhvcNAQEBBQAEggEAXcOmKWryntUYvis6Z5c2
# LrXBT1tidDe5vDf1mxlp5jijve6wGzjcsPvY8kR+pWmGLo0ep7HJYPtEdq1VMm4m
# 0RxWKNpAtH0WZGI038LPvu+QywclxAhBiH6l9bduDs5z2Lo58bnB9Cbto1gJWb5T
# TjKSS0jzhBswATWgBWeKmE6N2tUN/r+g9mc41AImC15F7JoaJiD8sjPRo+KNP9jj
# SO5qvPQ6PfGVFEXUzmuCFa2hxBpg0WdLxKS1W0ecKprknECiAUligTKbGbenK4qI
# wdDTkkyVRZAkQwV5/fRNqeaoRi4QdlY9jrV/+dNdEqrsSAP9vOPb3DVXT1DhEej5
# Dw==
# SIG # End signature block
